import pytest
from unittest import TestCase
from unittest.mock import patch

from hm_pyhelper.exceptions import UnknownVariantException, \
                                   UnknownVariantAttributeException

from hm_pyhelper.hardware_definitions import variant_definitions, \
                                            get_variant_attribute


class TestHardwareDefinitions(TestCase):
    def test_variant_definitions(self):
        # Not currently expecting APPNAME, GPIO_PIN_LED, or PIO_PIN_BUTTON
        expected_fields = {
            'FRIENDLY',
            'SPIBUS',
            'RESET',
            'MAC',
            'STATUS',
            'BUTTON',
            'ECCOB',
            'TYPE',
            'CELLULAR',
            'FCC_IDS',
            'CONTAINS_FCC_IDS',
            'IC_IDS',
            'CONTAINS_IC_IDS'
        }

        for variant_name, variant_dict in variant_definitions.items():
            variant_keys = variant_dict.keys()
            missing_keys = expected_fields.difference(variant_keys)
            self.assertSetEqual(missing_keys, set())

    mock_variant_definitions = {
        'NEBHNT-XYZ': {
            'FRIENDLY': 'XYZ Hotspot Gen 1',
            'APPNAME': 'APPNAMEXYZ',
            'SPIBUS': 'spidevX.Y',
            'KEY_STORAGE_BUS': '/dev/i2c-X',
            'RESET': 00,
            'MAC': 'ethXYZ',
            'STATUS': 00,
            'BUTTON': 00,
            'ECCOB': True,
            'TYPE': 'TYPEXYZ',
            'CELLULAR': False,
            'FCC_IDS': ['1'],
            'CONTAINS_FCC_IDS': ['2', '3'],
            'IC_IDS': ['4'],
            'CONTAINS_IC_IDS': []
        }
    }

    @patch('hm_pyhelper.hardware_definitions.variant_definitions',
           mock_variant_definitions)
    def test_get_variant_attribute(self):
        mock_variant = self.mock_variant_definitions['NEBHNT-XYZ']
        mock_variant_items = mock_variant.items()
        for attribute_name, attribute_val in mock_variant_items:
            returned_val = get_variant_attribute('NEBHNT-XYZ', attribute_name)
            self.assertEqual(returned_val, attribute_val)

    @patch('hm_pyhelper.hardware_definitions.variant_definitions',
           mock_variant_definitions)
    def test_get_variant_attribute_unknown_variant(self):
        with pytest.raises(UnknownVariantException):
            get_variant_attribute('Nonexistant', 'FRIENDLY')

    @patch('hm_pyhelper.hardware_definitions.variant_definitions',
           mock_variant_definitions)
    def test_get_variant_attribute_unknown_attribute(self):
        with pytest.raises(UnknownVariantAttributeException):
            get_variant_attribute('NEBHNT-XYZ', 'Nonexistant')



# Keys that are expected to be in diagnostics report.
ECC_KEY = 'ECC'
ONBOARDING_KEY = 'onboarding_key'
ETH_MAC_ADDRESS_KEY = 'eth_mac_address'
WIFI_MAC_ADDRESS_KEY = 'wifi_mac_address'
PUBLIC_KEY = 'public_key'
BLUETOOTH_KEY = 'bluetooth'
VARIANT_KEY = 'VARIANT'
FREQUENCY_KEY = 'FREQ'
SERIAL_NUMBER_KEY = 'serial_number'
BALENA_DEVICE_ID_KEY = 'BALENA_DEVICE_UUID'
DISK_IMAGE_KEY = 'disk_image'
LTE_KEY = 'lte'
PF_KEY = 'legacy_pass_fail'
LORA_KEY = 'lora'



"Test cases for util.gpg module."

import unittest
import json

from hm_pyhelper.util.pgp import get_payload_from_clearsigned_message


SAMPLE_CLEARSIGNED_MESSAGE = """
-----BEGIN PGP SIGNED MESSAGE-----
Hash: SHA512

{
  "key1": "value1",
  "key2": [1, 2, 3, 4]
}

-----BEGIN PGP SIGNATURE-----

iQGzBAEBCgAdFiEEfSH7Wj/J4hZxOpWqMy5fwG12lo0FAmIDUGEACgkQMy5fwG12
lo0WDwv+Nn8ffqY3oKOQv11eRkn+w4NcAp5XFqwYz0e5e+DEfjoYYTVAEgUdK1gF
W2u8Jed9rW710A6yYXHlIZWKSHsQ1sob5lK3R/r+/lFrXLYYwgFGBWJmph/wuiQW
OuZPpenOPNNjh37xxxxwxMj2kqUHKfJ489H2xOpPqpA4tWRKAspQCkv/AFVctcr5
1gWhJ5M5Mw/W6mgLswzonpsRy9M+vknuiDJ9F/Qe2hWUBJ0p7Si7YJkrynF3Oiqz
3D3JCiZAmoHq31hxO3bU7Sltf0lp1E7rG7mx7l0Pxq3fzNgaA9IW30NuF9J3YohK
Cv4pl11BYuUsBiQZktKcUwAsNj+UmZTnFBWACn4444+tFvs/tgG3wFZA33y3HAkl
6X9WGEa4d+DXeMgk2hr5oMRI9tZWqhNFqpu96CzrjJDxchgpeYjSCFiiX3po6Gyd
vH8uvB5hNKzj1vwqEtyyyyikUrPBe+273VeXNB+npF4LRok1MBjHZ49oZd2GZKBl
vo5u8szs
=i6qt
-----END PGP SIGNATURE-----
"""


class TestUtilPGP(unittest.TestCase):

    def test_empty_file(self):
        with self.assertRaises(RuntimeError) as exp:
            get_payload_from_clearsigned_message("")

        assert str(exp.exception) == \
            'Invalid message format, no --BEGIN PGP SIGNED MESSAGE-- header'

    def test_invalid_clearsigned_payload(self):
        with self.assertRaises(RuntimeError) as exp:
            get_payload_from_clearsigned_message("Just\nSome\nMessage\na\nb\nc")

        assert str(exp.exception) == \
            'Invalid message format, no --BEGIN PGP SIGNED MESSAGE-- header'

    def test_payload_extraction(self):
        payload = get_payload_from_clearsigned_message(SAMPLE_CLEARSIGNED_MESSAGE)
        d = json.loads(payload)
        assert d['key1'] == 'value1'
        assert d['key2'] == [1, 2, 3, 4]



class Diagnostic():
    """
    Pseudo-interface class that should be extended in order to add
    a type of diagnostic reading to a DiagnosticReport.
    """

    def __init__(self, key, friendly_key):
        """
        key - Key of relevant value in diagnostics_report dictionary
        friendly_key - Same as key but a human_friendly_snake_case
                        version. To replace key eventually.
        """
        self.key = key
        self.friendly_key = friendly_key

    def perform_test(self, diagnostics_report):
        raise Exception("Should be implemented by extending class")



import functools
import threading
from hm_pyhelper.logger import get_logger

LOGGER = get_logger(__name__)

DEFAULT_TIMEOUT = 2.0  # 2 seconds


class LockSingleton(object):
    _instance = None
    _lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            with cls._lock:
                if not cls._instance:
                    cls._instance = super(LockSingleton, cls).__new__(cls)
        return cls._instance

    def acquire(self, timeout=DEFAULT_TIMEOUT):
        if not self._lock.acquire(blocking=True, timeout=timeout):
            raise ResourceBusyError()

    def release(self):
        self._lock.release()

    def locked(self):
        return self._lock.locked()


class ResourceBusyError(Exception):
    """Raised when the resource is busy"""
    pass


def lock_ecc(timeout=DEFAULT_TIMEOUT, raise_resource_busy_exception=True):
    """
    Returns a decorator that locks the ECC.

    timeout: timeout value. DEFAULT_TIMEOUT = 2 seconds.
    raise_resource_busy_exception: set True to raise exception
                    in case of lock acquire timeout and error.
                    Otherwise just log the error msg
    """

    def decorator_lock_ecc(func):
        lock = LockSingleton()

        @functools.wraps(func)
        def wrapper_lock_ecc(*args, **kwargs):
            try:
                # try to acquire the ECC resource or may raise an exception
                lock.acquire(timeout=timeout)

                try:
                    value = func(*args, **kwargs)
                except Exception as ex:
                    lock.release()
                    raise ex

                # release the resource
                lock.release()

                return value
            except ResourceBusyError as ex:
                LOGGER.error("ECC is busy now.")
                if raise_resource_busy_exception:
                    raise ex
            except Exception as ex:
                raise ex

        return wrapper_lock_ecc

    return decorator_lock_ecc



import os
import subprocess
import json
from retry import retry
from hm_pyhelper.lock_singleton import ResourceBusyError, lock_ecc
from hm_pyhelper.logger import get_logger
from hm_pyhelper.exceptions import MalformedRegionException, \
    SPIUnavailableException, ECCMalfunctionException, \
    GatewayMFRFileNotFoundException, \
    MinerFailedToFetchMacAddress
from hm_pyhelper.miner_json_rpc.exceptions import \
     MinerFailedToFetchEthernetAddress
from hm_pyhelper.hardware_definitions import get_variant_attribute, \
    UnknownVariantException, UnknownVariantAttributeException


LOGGER = get_logger(__name__)
REGION_INVALID_SLEEP_SECONDS = 30
REGION_FILE_MISSING_SLEEP_SECONDS = 60
SPI_UNAVAILABLE_SLEEP_SECONDS = 60


@lock_ecc()
def run_gateway_mfr(args):
    direct_path = os.path.dirname(os.path.abspath(__file__))
    gateway_mfr_path = os.path.join(direct_path, 'gateway_mfr')

    command = [gateway_mfr_path]

    try:
        path_arg = [
            '--path',
            get_variant_attribute(os.getenv('VARIANT'), 'KEY_STORAGE_BUS')
        ]
        command.extend(path_arg)
    except (UnknownVariantException, UnknownVariantAttributeException) as e:
        LOGGER.warning(str(e) + ' Omitting --path arg.')

    command.extend(args)

    try:
        run_gateway_mfr_result = subprocess.run(
            command,
            capture_output=True,
            check=True
        )
        LOGGER.info(
            'gateway_mfr response stdout: %s' % run_gateway_mfr_result.stdout)
        LOGGER.info(
            'gateway_mfr response stderr: %s' % run_gateway_mfr_result.stderr)
    except subprocess.CalledProcessError as e:
        err_str = "gateway_mfr exited with a non-zero status"
        LOGGER.exception(err_str)
        raise ECCMalfunctionException(err_str).with_traceback(e.__traceback__)
    except (FileNotFoundError, NotADirectoryError) as e:
        err_str = "file/directory for gateway_mfr was not found"
        LOGGER.exception(err_str)
        raise GatewayMFRFileNotFoundException(err_str) \
            .with_traceback(e.__traceback__)
    except ResourceBusyError as e:
        err_str = "resource busy error: %s"
        LOGGER.exception(err_str % str(e))
        raise ResourceBusyError(e)\
            .with_traceback(e.__traceback__)
    except Exception as e:
        err_str = "Exception occured on running gateway_mfr %s" \
                  % str(e)
        LOGGER.exception(e)
        raise ECCMalfunctionException(err_str).with_traceback(e.__traceback__)

    try:
        return json.loads(run_gateway_mfr_result.stdout)
    except json.JSONDecodeError as e:
        err_str = "Unable to parse JSON from gateway_mfr"
        LOGGER.exception(err_str)
        raise ECCMalfunctionException(err_str).with_traceback(e.__traceback__)


def get_public_keys_rust():
    """
    Run gateway_mfr and report back the key.
    """
    return run_gateway_mfr(["key", "0"])


def get_getway_mfr_info():
    """
    Run gateway_mfr info.
    """
    return run_gateway_mfr(["info"])


def get_gateway_mfr_test_result():
    """
    Run gateway_mfr test and report back.
    """
    return run_gateway_mfr(["test"])


def provision_key():
    """
    Attempt to provision key.
    """
    test_results = get_gateway_mfr_test_result()
    if did_gateway_mfr_test_result_include_miner_key_pass(test_results):
        return True

    try:
        gateway_mfr_result = run_gateway_mfr(["provision"])
        LOGGER.info("[ECC Provisioning] %s", gateway_mfr_result)

    except subprocess.CalledProcessError:
        LOGGER.error("[ECC Provisioning] Exited with a non-zero status")
        return False
    return True


def did_gateway_mfr_test_result_include_miner_key_pass(
        gateway_mfr_test_result
):
    """
    Returns true if gateway_mfr_test_result["tests"] has an entry where
    "test": "miner_key(0)" and "result": "pass"
    Input: {
        "result": "pass",
        "tests": [
            {
            "output": "ok",
            "result": "pass",
            "test": "serial"
            },
            {
            "output": "ok",
            "result": "pass",
            "test": "zone_locked(data)"
            },
            {
            "output": "ok",
            "result": "pass",
            "test": "zone_locked(config)"
            },
            {
            "output": "ok",
            "result": "pass",
            "test": "slot_config(0..=15, ecc)"
            },
            {
            "output": "ok",
            "result": "pass",
            "test": "key_config(0..=15, ecc)"
            },
            {
            "output": "ok",
            "result": "pass",
            "test": "miner_key(0)"
            }
        ]
    }
    """

    def is_miner_key_and_passed(test_result):
        return test_result['test'] == 'miner_key(0)' and \
               test_result['result'] == 'pass'

    results_is_miner_key_and_passed = map(
        is_miner_key_and_passed,
        gateway_mfr_test_result['tests']
    )
    return any(results_is_miner_key_and_passed)


def get_ethernet_addresses(diagnostics):
    # Get ethernet MAC and WIFI address

    # The order of the values in the lists is important!
    # It determines which value will be available for which key
    path_to_files = [
        "/sys/class/net/eth0/address",
        "/sys/class/net/wlan0/address"
    ]
    keys = ["E0", "W0"]
    for (path, key) in zip(path_to_files, keys):
        try:
            diagnostics[key] = get_mac_address(path)
        except MinerFailedToFetchMacAddress as e:
            diagnostics[key] = False
            LOGGER.error(e)
        except Exception as e:
            diagnostics[key] = False
            LOGGER.error(e)
            raise MinerFailedToFetchEthernetAddress(str(e))


def get_mac_address(path):
    """
    input: path to the file with the location of the mac address
    output: A string containing a mac address
    Possible exceptions:
        FileNotFoundError - when the file is not found
        PermissionError - in the absence of access rights to the file
        TypeError - If the function argument is not a string.
    """
    if type(path) is not str:
        raise TypeError(
            "Constructing miner mac address failed.\
             The path must be a string value")
    try:
        file = open(path)
    except MinerFailedToFetchMacAddress as e:
        LOGGER.exception(str(e))
    except FileNotFoundError as e:
        LOGGER.exception("Failed to find Miner"
                         "Mac Address file at path %s" % path)
        raise MinerFailedToFetchMacAddress("Failed to find file"
                                           "containing miner mac address."
                                           "Exception: %s" % str(e))\
              .with_traceback(e.__traceback__)
    except PermissionError as e:
        LOGGER.exception("Permissions invalid for Miner"
                         "Mac Address file at path %s" % path)
        raise MinerFailedToFetchMacAddress("Failed to fetch"
                                           "miner mac address. "
                                           "Invalid permissions to access "
                                           "file. Exception: %s" % str(e))\
              .with_traceback(e.__traceback__)
    except Exception as e:
        LOGGER.exception(e)
        raise MinerFailedToFetchMacAddress("Failed to fetch miner"
                                           "mac address. "
                                           "Exception: %s" % str(e))\
              .with_traceback(e.__traceback__)

    return file.readline().strip().upper()


@retry(MalformedRegionException, delay=REGION_INVALID_SLEEP_SECONDS,
       logger=LOGGER)  # noqa
@retry(FileNotFoundError, delay=REGION_FILE_MISSING_SLEEP_SECONDS,
       logger=LOGGER)  # noqa
def retry_get_region(region_override, region_filepath):
    """
    Return the override if it exists, or parse file created by hm-miner.
    region_override is the actual value,
    not the name of the environment variable.
    Retry if region in file is malformed or not found.
    """
    if region_override:
        return region_override

    LOGGER.debug(
        "No region override set (value = %s), will retrieve from miner." % region_override)  # noqa: E501
    with open(region_filepath) as region_file:
        region = region_file.read().rstrip('\n')
        LOGGER.debug("Region %s parsed from %s " % (region, region_filepath))

        is_region_valid = len(region) > 3
        if is_region_valid:
            return region

        raise MalformedRegionException("Region %s is invalid" % region)


@retry(SPIUnavailableException, delay=SPI_UNAVAILABLE_SLEEP_SECONDS,
       logger=LOGGER)  # noqa
def await_spi_available(spi_bus):
    """
    Check that the SPI bus path exists, assuming it is in /dev/{spi_bus}
    """
    if os.path.exists('/dev/{}'.format(spi_bus)):
        LOGGER.debug("SPI bus %s Configured Correctly" % spi_bus)
        return True
    else:
        raise SPIUnavailableException("SPI bus %s not found!" % spi_bus)



"""Utility code to work with GPG signed messages."""


def get_payload_from_clearsigned_message(message: str) -> str:
    """
    Given a message in clearsign format removes signature and marker strings like
    -----BEGIN PGP SIGNATURE-----, -----BEGIN PGP SIGNED MESSAGE----- to extract
    the original payload.

    :param message: The message containing the signature and the payload.
    :return: Extracted payload as string. Calling code is responsible for
        converting it to proper data type.
    """
    lines = message.strip().split('\n')

    if len(lines) < 5 \
            or lines[0] != '-----BEGIN PGP SIGNED MESSAGE-----' \
            or lines[1].startswith('Hash:') is False:
        raise RuntimeError("Invalid message format, no --BEGIN PGP SIGNED MESSAGE-- header")

    start_idx = 3  # Payload starts from 3rd line in clearsigned messages
    end_idx = None

    for idx, line in enumerate(lines[3:]):
        if line.strip() == '-----BEGIN PGP SIGNATURE-----':
            end_idx = idx + start_idx
            break

    if end_idx is None:
        raise RuntimeError("Invalid message format, no --BEGIN PGP SIGNATURE-- section")

    return "\n".join(lines[start_idx: end_idx])



import unittest
import mock
import responses
import requests
from hm_pyhelper.miner_json_rpc import MinerClient
from hm_pyhelper.miner_json_rpc.exceptions import MinerRegionUnset
from hm_pyhelper.miner_json_rpc.exceptions import MinerMalformedURL
from hm_pyhelper.miner_json_rpc.exceptions import MinerConnectionError

BASE_URL = 'http://helium-miner:4467'


@responses.activate
def response_result(data, status):
    url = "https://fake_url"
    responses.add(responses.POST, url, json=data, status=status)
    resp = requests.post(url)
    print(resp.json())
    return resp


def return_payload_with_method(method):
    return {'jsonrpc': '2.0', 'id': 1, 'method': method}


class Result(object):
    def __init__(self, result={'my': 'data'}):
        self.result = result


class Response(object):
    def __init__(self, data=Result()):
        self.data = data


class TestMinerJSONRPC(unittest.TestCase):
    def test_instantiation(self):
        client = MinerClient()
        self.assertIsInstance(client, MinerClient)
        self.assertEqual(client.url, BASE_URL)

    def test_malformed_url(self):
        client = MinerClient(url='fakeurl')

        exception_raised = False
        exception_type = None
        try:
            client.get_height()
        except Exception as exc:
            exception_raised = True
            exception_type = exc

        self.assertTrue(exception_raised)
        self.assertIsInstance(exception_type, MinerMalformedURL)

    def test_connection_error(self):
        client = MinerClient(url='http://notarealminer:9999')

        exception_raised = False
        exception_type = None
        try:
            client.get_height()
        except Exception as exc:
            exception_raised = True
            exception_type = exc

        self.assertTrue(exception_raised)
        self.assertIsInstance(exception_type, MinerConnectionError)

    @mock.patch('hm_pyhelper.miner_json_rpc.client.requests.post',
                return_value=response_result(
                    {"result": {'epoch': 25612, 'height': 993640}, "id": 1},
                    200))
    def test_get_height(self, mock_json_rpc_client):
        client = MinerClient()
        result = client.get_height()
        mock_json_rpc_client.assert_called_with(
            BASE_URL, json=return_payload_with_method('info_height'))

        self.assertEqual(result, {'epoch': 25612, 'height': 993640})

    @mock.patch('hm_pyhelper.miner_json_rpc.client.requests.post',
                return_value=response_result(
                    {"result": {'region': None}, "id": 1}, 200))
    def test_get_region_not_asserted(self, mock_json_rpc_client):
        client = MinerClient()
        exception_raised = False
        exception_type = None

        try:
            client.get_region()
        except Exception as exc:
            exception_raised = True
            exception_type = exc

        self.assertTrue(exception_raised)
        self.assertIsInstance(exception_type, MinerRegionUnset)

    @mock.patch('hm_pyhelper.miner_json_rpc.client.requests.post',
                return_value=response_result(
                    {"result": {'region': "EU868"}, "id": 1}, 200))
    def test_get_region(self, mock_json_rpc_client):
        client = MinerClient()
        result = client.get_region()
        mock_json_rpc_client.assert_called_with(
            BASE_URL, json=return_payload_with_method('info_region')
        )
        self.assertEqual(result, {'region': 'EU868'})

    summary = {
        'block_age': 1136610,
        'epoch': 25612,
        'firmware_version': "0.1",
        'gateway_details': 'undefined',
        'height': 993640,
        'mac_addresses': [
            {'eth0': '0242AC110002'},
            {'ip6tnl0': '00000000000000000000000000000000'},
            {'tunl0': '00000000'},
            {'lo': '000000000000'}
        ],
        'name': 'scruffy-chocolate-shell',
        'peer_book_entry_count': 3,
        'sync_height': 993640,
        'uptime': 144,
        'version': 10010005
    }

    result_json = {"result": summary, "id": 1}

    @mock.patch('hm_pyhelper.miner_json_rpc.client.requests.post',
                return_value=response_result(result_json, 200))
    def test_get_summary(self, mock_json_rpc_client):
        client = MinerClient()
        result = client.get_summary()
        mock_json_rpc_client.assert_called_with(
            BASE_URL, json=return_payload_with_method('info_summary')
        )
        self.assertEqual(result, self.summary)

    peer_addr = '/p2p/11jr2kMp1bZvSC6pd3XkNvs9Q43qCgEzxRwV6vpuqXanC5UcLEs'

    @mock.patch('hm_pyhelper.miner_json_rpc.client.requests.post',
                return_value=response_result(
                    {"result": {'peer_addr': peer_addr}, "id": 1}, 200))
    def test_get_peer_addr(self, mock_json_rpc_client):

        client = MinerClient()
        result = client.get_peer_addr()
        mock_json_rpc_client.assert_called_with(
            BASE_URL, json=return_payload_with_method('peer_addr')
        )
        self.assertEqual(result, {'peer_addr': self.peer_addr})

    @mock.patch('hm_pyhelper.miner_json_rpc.client.requests.post',
                return_value=response_result(
                    {"result": [], "id": 1},
                    200))
    def test_get_peer_book(self, mock_json_rpc_client):

        client = MinerClient()
        result = client.get_peer_book()
        payload = return_payload_with_method('peer_book')
        payload["params"] = {'addr': 'self'}
        mock_json_rpc_client.assert_called_with(
            BASE_URL, json=payload
        )
        self.assertEqual(result, [])

    firmware_version = '2021.10.18.0'
    data_response = {
        'block_age': 1136610,
        'epoch': 25612,
        'firmware_version': firmware_version,
        'gateway_details': 'undefined',
        'height': 993640,
        'mac_addresses': [
            {'eth0': '0242AC110002'},
            {'ip6tnl0': '00000000000000000000000000000000'},
            {'tunl0': '00000000'},
            {'lo': '000000000000'}
        ],
        'name': 'scruffy-chocolate-shell',
        'peer_book_entry_count': 3,
        'sync_height': 993640,
        'uptime': 144,
        'version': 10010005
    }

    result_response = {"result": data_response, "id": 1}

    @mock.patch('hm_pyhelper.miner_json_rpc.client.requests.post',
                return_value=response_result(
                    result_response, 200))
    def test_get_firmware_version(self, mock_json_rpc_client):
        client = MinerClient()
        result = client.get_firmware_version()
        mock_json_rpc_client.assert_called_with(
            BASE_URL, json=return_payload_with_method('info_summary')
        )
        self.assertEqual(result, self.firmware_version)

    add_gateway_response = {
        "jsonrpc": "2.0",
        "result": {
            "result": "CroBCiEBwPbb63LQD8x/m/ZDLLyOLgtxypQIjh+xPPS+d8g/i24SIQB"
                      "8XdzWqrIF201DNKHpXKFtsMtgvZeqBBc1wOk9sV2j4SJGMEQCIGE82g"
                      "Hbn0z/AOyaDXsuQDptC/I15fHCF//QEgzoxodrAiBsoRUiw8zMVttkP"
                      "hEOoMfM0smmdCZPKX6tVOgK0s/0KiohAVHXaw1kNly2VOt47MlzTfkC"
                      "IUTOgW34Orw1LSJt+9mCOICS9AFA6PsD"
        },
        "id": "1"
    }

    @mock.patch('hm_pyhelper.miner_json_rpc.client.requests.post',
                return_value=response_result(
                    add_gateway_response, 200))
    def test_create_add_gateway_txn(self, mock_json_rpc_client):
        self.maxDiff = None
        client = MinerClient()
        actual_result = client.create_add_gateway_txn(
            '14QjC3A5DEH2uFwhDxyBHdFir7YWGG23Fic1wGvkCr6qrWC7Q47',
            '13Zni1he7KY9pUmkXMhEhTwfUpL9AcEV1m2UbbvFsrU9QPTMgE3')

        expected_result = {
            'gateway_address':
            '11wmnCAfvFkdx3Az1hesTsSv9YeBUhs8JZKjCqcs2vmRwRazpBa',

            'owner_address':
            '14QjC3A5DEH2uFwhDxyBHdFir7YWGG23Fic1wGvkCr6qrWC7Q47',

            'payer_address':
            '13Zni1he7KY9pUmkXMhEhTwfUpL9AcEV1m2UbbvFsrU9QPTMgE3',

            'fee': 65000,
            'staking_fee': 4000000,
            'txn': "CroBCiEBwPbb63LQD8x/m/ZDLLyOLgtxypQIjh+xPPS+d8g/i24SIQB8Xd"
                   "zWqrIF201DNKHpXKFtsMtgvZeqBBc1wOk9sV2j4SJGMEQCIGE82gHbn0z/"
                   "AOyaDXsuQDptC/I15fHCF//QEgzoxodrAiBsoRUiw8zMVttkPhEOoMfM0s"
                   "mmdCZPKX6tVOgK0s/0KiohAVHXaw1kNly2VOt47MlzTfkCIUTOgW34Orw1"
                   "LSJt+9mCOICS9AFA6PsD"
        }

        self.assertDictEqual(actual_result, expected_result)
